@@ -3,7 +3,7 @@
 // Purpose:     
 // Author:      Alex Thuering
 // Created:     2005/05/09
-// RCS-ID:      $Id: SVGCanvasItem.cpp,v 1.42 2013/01/19 18:26:28 ntalex Exp $
+// RCS-ID:      $Id: SVGCanvasItem.cpp,v 1.44 2013/03/31 14:10:45 ntalex Exp $
 // Copyright:   (c) 2005 Alex Thuering
 // Licence:     wxWindows licence
 //////////////////////////////////////////////////////////////////////////////
@@ -19,6 +19,8 @@
 #include <wxSVG/mediadec_ffmpeg.h>
 #endif
 
+#undef GetCurrentTime
+
 //////////////////////////////////////////////////////////////////////////////
 /////////////////////////////// wxSVGCanvasPath //////////////////////////////
 //////////////////////////////////////////////////////////////////////////////
@@ -532,7 +534,10 @@ wxSVGCanvasText::wxSVGCanvasText(wxSVGCanvas* canvas) :
 	m_char = NULL;
 	m_tx = m_ty = 0;
 	m_textAnchor = wxCSS_VALUE_START;
+	m_textAnchorBeginIndex = 0;
+	m_textAnchorBeginPos = 0;
 	m_dominantBaseline = wxCSS_VALUE_AUTO;
+	m_dominantBaselineBeginIndex = 0;
 }
 
 wxSVGCanvasText::~wxSVGCanvasText() {
@@ -964,8 +969,10 @@ wxSVGCanvasVideoData::~wxSVGCanvasVideoData() {
 #endif
 }
 
-wxSVGCanvasVideo::wxSVGCanvasVideo(): wxSVGCanvasImage(wxSVG_CANVAS_ITEM_VIDEO), m_videoData(NULL) {
-	// nothing to do
+wxSVGCanvasVideo::wxSVGCanvasVideo(): wxSVGCanvasImage(wxSVG_CANVAS_ITEM_VIDEO) {
+	m_time = 0;
+	m_duration = 0;
+	m_videoData = NULL;
 }
 
 wxSVGCanvasVideo::~wxSVGCanvasVideo() {
@@ -997,16 +1004,18 @@ void wxSVGCanvasVideo::Init(wxSVGVideoElement& element, const wxCSSStyleDeclarat
 		m_defHeightScale = prevItem->m_defHeightScale;
 		wxFfmpegMediaDecoder* decoder = m_videoData->GetMediaDecoder();
 		if (decoder != NULL) {
-			double ftime = decoder->GetFps() != -1 ? 1.0 / decoder->GetFps() : 0.04;
+			double ftime = decoder->GetFps() >= 1 ? 1.0 / decoder->GetFps() : 0.04;
 			double currTime = decoder->GetPosition();
-			if (currTime != m_time && (m_time < currTime || m_time - currTime >= ftime - 0.001)) {
-				if (m_time < currTime || m_time - currTime > 50*ftime)
+			if (currTime != m_time && (currTime >= m_time + ftime/2 || m_time - currTime > ftime/2)) {
+				if (currTime > m_time || m_time - currTime > 50*ftime) {
 					decoder->SetPosition(m_time > 1.0 ? m_time - 1.0 : 0.0);
+				}
 				for (int i = 0; i < 60; i++) {
 					m_image = decoder->GetNextFrame();
 					currTime = decoder->GetPosition();
-					if (currTime >= m_time - 0.001 || currTime < 0)
+					if (currTime >= m_time - ftime/2 || currTime < 0) {
 						break;
+					}
 				}
 			} else
 				m_image = prevItem->m_image;
